#include <string.h>
#include "geosim.h"
#include "gspriv.h"
#include "infprocs.h"
#include "ietypes.h"
#include "iefile.h"
#include "editrec.h"
#include "windfunc.h"
#include "helpmsg.h"

BigString filename, templatename;
boolean existsOnDisk;
boolean confirmActions = TRUE;

extern boolean filewind, confirmwind;
EditRec *filenamerec = NULL;
GSColor redCol, blackCol;
char confirmText[100];

extern char *curEditItem;

#define CLOSE 0
#define REVERT 1
#define QUIT 2
#define NOTIFY 3
static int confirmState;

/* Internal protos */
boolean close_interface(boolean);
void SetupForNotify();
void char_typed(void);
/**/

boolean new_interface(void)
{
	if (strlen(filename) == 0) {
		GSdisablebutton("File", "New", TRUE);
		return;
		}

	if (!read_interface(templatename))	{
		/*filenamerec->inspt = 0;*/
		close_file(); /* +++ will this deallocate properly? */
		reset_state();
		/*UpdateEditRec(filenamerec);*/
		sprintf(confirmText, "Couldn't open template file %s.", templatename);
		SetupForNotify();
		DrawConfirmWind();
		}
	else	{
		GSdisablemenuitem("File", "Close");
		RemoveFileWind();
		/*DrawMiscWind();*/
		existsOnDisk = FALSE;
		DrawInterfaceWind();
		}
}

boolean open_interface(void)
{
	if (strlen(filename) == 0) {
		GSdisablebutton("File", "Open", TRUE);
		return;
		}
	if (!read_interface(filename))	{
		GSelog(0, "Couldn't read interface %s\n", filename);
		/*filenamerec->inspt = 0;*/
		close_file(); /* +++ will this deallocate properly? */
		reset_state();
		/*UpdateEditRec(filenamerec);*/
		sprintf(confirmText, "Couldn't open interface file %s.", filename);
		SetupForNotify();
		DrawConfirmWind();
		}
	else	{
		GSenablemenuitem("File", "Close");
		RemoveFileWind();
/*		GStogglecheck("Edit", E_MISC);*/
		/*DrawMiscWind();*/
		existsOnDisk = TRUE;
		DrawInterfaceWind();
		}
}

boolean save_interface(void)
{
	if (!write_interface(filename)) {
	/*	GSbell();*/
		return FALSE;
		}
	return TRUE;
	existsOnDisk = TRUE;
}

boolean close_interface(boolean save)
{
	remove_active_window();
	if (save)
		save_interface();
	close_file();
	reset_state();
/*	DrawFileWind();*/
}

boolean revert_interface(void)
{
	boolean success;
	
	remove_active_window();
	close_file();
	reset_state();
	
	success = read_interface(existsOnDisk? filename : templatename);
	
	if (!success) {
		close_file();
		reset_state();
		GSelog(0, "Couldn't revert to file |%s|.\n",
								existsOnDisk? filename : templatename);
		DrawFileWind();
		}
	else
		DrawMiscWind();
}

boolean quit(void)
{
	GSquit(FALSE);
}

boolean ask_close_interface()
{
	if (confirmActions) {
		GSenablebutton("Confirm", "No", FALSE);
		GSenablebutton("Confirm", "Yes", FALSE);
		GSenablebutton("Confirm", "Cancel", FALSE);
		GSinvisiblebutton("Confirm", "OK", FALSE);
		sprintf(confirmText, "Save interface %s before closing?", filename);
		DrawConfirmWind();
		confirmState = CLOSE;
		}
	else {
		close_interface(TRUE);
		DrawFileWind();
		}
}

boolean ask_revert_interface()
{
	if (confirmActions) {
		GSinvisiblebutton("Confirm", "No", FALSE);
		GSenablebutton("Confirm", "Yes", FALSE);
		GSenablebutton("Confirm", "Cancel", FALSE);
		GSinvisiblebutton("Confirm", "OK", FALSE);
		sprintf(confirmText, "Revert to saved version of %s?", filename);
		DrawConfirmWind();
		confirmState = REVERT;
		}
	else
		revert_interface();
}

boolean ask_quit()
{
	if (confirmActions && !filewind) {
		GSenablebutton("Confirm", "No", FALSE);
		GSenablebutton("Confirm", "Yes", FALSE);
		GSenablebutton("Confirm", "Cancel", FALSE);
		GSinvisiblebutton("Confirm", "OK", FALSE);
		sprintf(confirmText, "Save interface %s before quitting?", filename);
		DrawConfirmWind();
		confirmState = QUIT;
		}
	else {
		close_interface(!filewind);
		quit();
		}
}

boolean action_confirmed()
{
	RemoveConfirmWind();
	
	switch(confirmState) {
	
		case CLOSE:
			close_interface(TRUE);
			DrawFileWind();
			break;
			
		case REVERT:
			revert_interface();
			break;
			
		case QUIT:
			close_interface(TRUE);
			quit();
			break;
		
		case NOTIFY:
			break;
			}
}

boolean action_denied()
{
	RemoveConfirmWind();
	
	switch(confirmState) {
	
		case CLOSE:
			close_interface(FALSE);
			DrawFileWind();
			break;
			
		case QUIT:
			quit();
			break;
		}
}

boolean action_canceled()
{
	RemoveConfirmWind();
}

void char_typed(void)
{
	ButtonStatusType bstat;
	int name_len = strlen(filename);

	GSgetbuttonspecs("File", "New",
			NULL, NULL, NULL, NULL,
			&bstat);

	if ((bstat == ENABLED) && (name_len == 0)) {
		GSdisablebutton("File", "New", TRUE);
		GSdisablebutton("File", "Open", TRUE);
		}
	else if ((bstat == DISABLED) && (name_len > 0)) {
		GSenablebutton("File", "New", TRUE);
		GSenablebutton("File", "Open", TRUE);
		}
}

void DrawFileWind()
{
	redCol = GSconvertcolor("red");
	blackCol = GSconvertcolor("black");

	*filename = '\0';
	GSdisablebutton("File", "New", FALSE);
	GSdisablebutton("File", "Open", FALSE);
	GSdrawnamedwindow("File", NO_BSTORE, NO_POPUP);	
	filenamerec = NewEditRec(filename, BIGSTRINGLEN, 5, SMALL,
		"File", "filenamesel", NULL, char_typed);
	SetCurrentEditRec(filenamerec);
	filewind = TRUE;
	GSdisablemenuitem("File", "save");
	GSdisablemenuitem("File", "close");
	GSdisablemenuitem("File", "revert");
	GSdisablemenuitem("Edit", "edit_misc");
	GSdisablemenuitem("Edit", "edit_palette");
	GSdisablemenuitem("Edit", "edit_messages");
	GSdisablemenuitem("Edit", "edit_functions");
	GSdisablemenuitem("Edit", "edit_aliases");
	GSdisablemenuitem("Edit", "edit_interface");
	GSdisplayhelpmessage(IE_INTRO);
	ShowVersion();
}

void RedrawFileWind()
{
	GSredrawnamedwindow("File");
	DrawEditRec(filenamerec);
	GSdisplayhelpmessage(-1);
}

void RemoveFileWind()
{
	GSremovenamedwindow("File");
	DisposEditRec(filenamerec);
	filenamerec = NULL;
	filewind = FALSE;
	GSredrawnamedwindow("root");
	GSredrawnamedwindow("message");
	GSenablemenuitem("File", "save");
	GSenablemenuitem("File", "close");
	GSenablemenuitem("File", "revert");
	GSenablemenuitem("Edit", "edit_misc");
	GSenablemenuitem("Edit", "edit_palette");
	GSenablemenuitem("Edit", "edit_messages");
	GSenablemenuitem("Edit", "edit_functions");
	GSenablemenuitem("Edit", "edit_aliases");
	GSenablemenuitem("Edit", "edit_interface");
}

void DrawConfirmWind()
{
	confirmwind = TRUE;
	GSdrawnamedwindow("Confirm", BSTORE, POPUP);
	RedrawConfirmWind();
}

void RedrawConfirmWind()
{
	int x, y, wd, ht;
	GSredrawnamedwindow("Confirm");
	GSgetfieldrect("ConfirmMessage", &x, &y, &wd, &ht, NULL);
	GSsetcolor(blackCol);
	GSsetfont(LARGE);
	GSwwriteclippedmsg(x, y, wd, ht, x, y+LARGE+1, confirmText);
}

void RemoveConfirmWind()
{
	confirmwind = FALSE;
	GSremovenamedwindow("Confirm");
}

void SetupForNotify()
{
	GSinvisiblebutton("Confirm", "No", FALSE);
	GSinvisiblebutton("Confirm", "Yes", FALSE);
	GSinvisiblebutton("Confirm", "Cancel", FALSE);
	GSenablebutton("Confirm", "OK", FALSE);
	confirmState = NOTIFY;
}
